@@ -5,7 +5,7 @@ package IkiWiki::Plugin::inline;
 use warnings;
 use strict;
 use Encode;
-use IkiWiki 2.00;
+use IkiWiki 3.00;
 use URI;
 
 my %knownfeeds;
@@ -13,24 +13,23 @@ my %page_numfeeds;
 my @inline;
 my $nested=0;
 
-sub import { #{{{
+sub import {
 	hook(type => "getopt", id => "inline", call => \&getopt);
+	hook(type => "getsetup", id => "inline", call => \&getsetup);
 	hook(type => "checkconfig", id => "inline", call => \&checkconfig);
 	hook(type => "sessioncgi", id => "inline", call => \&sessioncgi);
 	hook(type => "preprocess", id => "inline", 
 		call => \&IkiWiki::preprocess_inline);
 	hook(type => "pagetemplate", id => "inline",
 		call => \&IkiWiki::pagetemplate_inline);
-	hook(type => "format", id => "inline", call => \&format);
+	hook(type => "format", id => "inline", call => \&format, first => 1);
 	# Hook to change to do pinging since it's called late.
 	# This ensures each page only pings once and prevents slow
 	# pings interrupting page builds.
-	hook(type => "change", id => "inline", 
-		call => \&IkiWiki::pingurl);
+	hook(type => "change", id => "inline", call => \&IkiWiki::pingurl);
+}
 
-} # }}}
-
-sub getopt () { #{{{
+sub getopt () {
 	eval q{use Getopt::Long};
 	error($@) if $@;
 	Getopt::Long::Configure('pass_through');
@@ -39,10 +38,56 @@ sub getopt () { #{{{
 		"atom!" => \$config{atom},
 		"allowrss!" => \$config{allowrss},
 		"allowatom!" => \$config{allowatom},
+		"pingurl=s" => sub {
+			push @{$config{pingurl}}, $_[1];
+		},      
 	);
 }
 
-sub checkconfig () { #{{{
+sub getsetup () {
+	return
+		plugin => {
+			safe => 1,
+			rebuild => undef,
+		},
+		rss => {
+			type => "boolean",
+			example => 0,
+			description => "enable rss feeds by default?",
+			safe => 1,
+			rebuild => 1,
+		},
+		atom => {
+			type => "boolean",
+			example => 0,
+			description => "enable atom feeds by default?",
+			safe => 1,
+			rebuild => 1,
+		},
+		allowrss => {
+			type => "boolean",
+			example => 0,
+			description => "allow rss feeds to be used?",
+			safe => 1,
+			rebuild => 1,
+		},
+		allowatom => {
+			type => "boolean",
+			example => 0,
+			description => "allow atom feeds to be used?",
+			safe => 1,
+			rebuild => 1,
+		},
+		pingurl => {
+			type => "string",
+			example => "http://rpc.technorati.com/rpc/ping",
+			description => "urls to ping (using XML-RPC) on feed update",
+			safe => 1,
+			rebuild => 0,
+		},
+}
+
+sub checkconfig () {
 	if (($config{rss} || $config{atom}) && ! length $config{url}) {
 		error(gettext("Must specify url to wiki with --url when using --rss or --atom"));
 	}
@@ -52,9 +97,12 @@ sub checkconfig () { #{{{
 	if ($config{atom}) {
 		push @{$config{wiki_file_prune_regexps}}, qr/\.atom$/;
 	}
-} #}}}
+	if (! exists $config{pingurl}) {
+		$config{pingurl}=[];
+	}
+}
 
-sub format (@) { #{{{
+sub format (@) {
         my %params=@_;
 
 	# Fill in the inline content generated earlier. This is actually an
@@ -63,14 +111,14 @@ sub format (@) { #{{{
 		delete @inline[$1,]
 	}eg;
 	return $params{content};
-} #}}}
+}
 
-sub sessioncgi () { #{{{
+sub sessioncgi ($$) {
 	my $q=shift;
 	my $session=shift;
 
 	if ($q->param('do') eq 'blog') {
-		my $page=IkiWiki::titlepage(decode_utf8($q->param('title')));
+		my $page=titlepage(decode_utf8($q->param('title')));
 		$page=~s/(\/)/"__".ord($1)."__"/eg; # don't create subdirs
 		# if the page already exists, munge it to be unique
 		my $from=$q->param('from');
@@ -82,7 +130,13 @@ sub sessioncgi () { #{{{
 		$q->param('page', $page.$add);
 		# now go create the page
 		$q->param('do', 'create');
-		IkiWiki::cgi_editpage($q, $session);
+		# make sure the editpage plugin in loaded
+		if (IkiWiki->can("cgi_editpage")) {
+			IkiWiki::cgi_editpage($q, $session);
+		}
+		else {
+			error(gettext("page editing not allowed"));
+		}
 		exit;
 	}
 }
@@ -94,7 +148,7 @@ package IkiWiki;
 my %toping;
 my %feedlinks;
 
-sub preprocess_inline (@) { #{{{
+sub preprocess_inline (@) {
 	my %params=@_;
 	
 	if (! exists $params{pages}) {
@@ -106,6 +160,7 @@ sub preprocess_inline (@) { #{{{
 	my $atom=(($config{atom} || $config{allowatom}) && exists $params{atom}) ? yesno($params{atom}) : $config{atom};
 	my $quick=exists $params{quick} ? yesno($params{quick}) : 0;
 	my $feeds=exists $params{feeds} ? yesno($params{feeds}) : !$quick;
+	my $emptyfeeds=exists $params{emptyfeeds} ? yesno($params{emptyfeeds}) : 1;
 	my $feedonly=yesno($params{feedonly});
 	if (! exists $params{show} && ! $archive) {
 		$params{show}=10;
@@ -128,16 +183,19 @@ sub preprocess_inline (@) { #{{{
 		$params{template} = $archive ? "archivepage" : "inlinepage";
 	}
 
-	my @list;
-	foreach my $page (keys %pagesources) {
-		next if $page eq $params{page};
-		if (pagespec_match($page, $params{pages}, location => $params{page})) {
-			push @list, $page;
-		}
-	}
+	my @list=pagespec_match_list(
+		[ grep { $_ ne $params{page} } keys %pagesources ],
+		$params{pages}, location => $params{page});
 
 	if (exists $params{sort} && $params{sort} eq 'title') {
-		@list=sort @list;
+		@list=sort { pagetitle(basename($a)) cmp pagetitle(basename($b)) } @list;
+	}
+	elsif (exists $params{sort} && $params{sort} eq 'title_natural') {
+		eval q{use Sort::Naturally};
+		if ($@) {
+			error(gettext("Sort::Naturally needed for title_natural sort"));
+		}
+		@list=sort { Sort::Naturally::ncmp(pagetitle(basename($a)), pagetitle(basename($b))) } @list;
 	}
 	elsif (exists $params{sort} && $params{sort} eq 'mtime') {
 		@list=sort { $pagemtime{$b} <=> $pagemtime{$a} } @list;
@@ -146,7 +204,7 @@ sub preprocess_inline (@) { #{{{
 		@list=sort { $pagectime{$b} <=> $pagectime{$a} } @list;
 	}
 	else {
-		return sprintf(gettext("unknown sort type %s"), $params{sort});
+		error sprintf(gettext("unknown sort type %s"), $params{sort});
 	}
 
 	if (yesno($params{reverse})) {
@@ -177,38 +235,70 @@ sub preprocess_inline (@) { #{{{
 	# that if they are removed or otherwise changed, the inline will be
 	# sure to be updated.
 	add_depends($params{page}, join(" or ", $#list >= $#feedlist ? @list : @feedlist));
-
-	my $feednum="";
-
-	my $feedid=join("\0", map { $_."\0".$params{$_} } sort keys %params);
-	if (exists $knownfeeds{$feedid}) {
-		$feednum=$knownfeeds{$feedid};
+	
+	if ($feeds && exists $params{feedpages}) {
+		@feedlist=grep { pagespec_match($_, $params{feedpages}, location => $params{page}) } @feedlist;
 	}
-	else {
-		if (exists $page_numfeeds{$params{destpage}}) {
-			if ($feeds) {
-				$feednum=$knownfeeds{$feedid}=++$page_numfeeds{$params{destpage}};
+
+	my ($feedbase, $feednum);
+	if ($feeds) {
+		# Ensure that multiple feeds on a page go to unique files.
+		
+		# Feedfile can lead to conflicts if usedirs is not enabled,
+		# so avoid supporting it in that case.
+		delete $params{feedfile} if ! $config{usedirs};
+		# Tight limits on legal feedfiles, to avoid security issues
+		# and conflicts.
+		if (defined $params{feedfile}) {
+			if ($params{feedfile} =~ /\// ||
+			    $params{feedfile} !~ /$config{wiki_file_regexp}/) {
+				error("illegal feedfile");
 			}
+			$params{feedfile}=possibly_foolish_untaint($params{feedfile});
+		}
+		$feedbase=targetpage($params{destpage}, "", $params{feedfile});
+
+		my $feedid=join("\0", $feedbase, map { $_."\0".$params{$_} } sort keys %params);
+		if (exists $knownfeeds{$feedid}) {
+			$feednum=$knownfeeds{$feedid};
 		}
 		else {
-			$feednum=$knownfeeds{$feedid}="";
-			if ($feeds) {
-				$page_numfeeds{$params{destpage}}=1;
+			if (exists $page_numfeeds{$params{destpage}}{$feedbase}) {
+				if ($feeds) {
+					$feednum=$knownfeeds{$feedid}=++$page_numfeeds{$params{destpage}}{$feedbase};
+				}
+			}
+			else {
+				$feednum=$knownfeeds{$feedid}="";
+				if ($feeds) {
+					$page_numfeeds{$params{destpage}}{$feedbase}=1;
+				}
 			}
 		}
 	}
 
-	my $rssurl=basename(rsspage($params{destpage}).$feednum) if $feeds && $rss;
-	my $atomurl=basename(atompage($params{destpage}).$feednum) if $feeds && $atom;
+	my $rssurl=abs2rel($feedbase."rss".$feednum, dirname(htmlpage($params{destpage}))) if $feeds && $rss;
+	my $atomurl=abs2rel($feedbase."atom".$feednum, dirname(htmlpage($params{destpage}))) if $feeds && $atom;
+
 	my $ret="";
 
-	if ($config{cgiurl} && ! $params{preview} && (exists $params{rootpage} ||
-			(exists $params{postform} && yesno($params{postform})))) {
+	if (length $config{cgiurl} && ! $params{preview} && (exists $params{rootpage} ||
+	    (exists $params{postform} && yesno($params{postform}))) &&
+	    IkiWiki->can("cgi_editpage")) {
 		# Add a blog post form, with feed buttons.
 		my $formtemplate=template("blogpost.tmpl", blind_cache => 1);
 		$formtemplate->param(cgiurl => $config{cgiurl});
-		$formtemplate->param(rootpage => 
-			exists $params{rootpage} ? $params{rootpage} : $params{page});
+		my $rootpage;
+		if (exists $params{rootpage}) {
+			$rootpage=bestlink($params{page}, $params{rootpage});
+			if (!length $rootpage) {
+				$rootpage=$params{rootpage};
+			}
+		}
+		else {
+			$rootpage=$params{page};
+		}
+		$formtemplate->param(rootpage => $rootpage);
 		$formtemplate->param(rssurl => $rssurl) if $feeds && $rss;
 		$formtemplate->param(atomurl => $atomurl) if $feeds && $atom;
 		if (exists $params{postformtext}) {
@@ -220,8 +310,12 @@ sub preprocess_inline (@) { #{{{
 				gettext("Add a new post titled:"));
 		}
 		$ret.=$formtemplate->output;
+	    	
+		# The post form includes the feed buttons, so
+		# emptyfeeds cannot be hidden.
+		$emptyfeeds=1;
 	}
-	elsif ($feeds && !$params{preview}) {
+	elsif ($feeds && !$params{preview} && ($emptyfeeds || @feedlist)) {
 		# Add feed buttons.
 		my $linktemplate=template("feedlink.tmpl", blind_cache => 1);
 		$linktemplate->param(rssurl => $rssurl) if $rss;
@@ -233,7 +327,7 @@ sub preprocess_inline (@) { #{{{
 		require HTML::Template;
 		my @params=IkiWiki::template_params($params{template}.".tmpl", blind_cache => 1);
 		if (! @params) {
-			return sprintf(gettext("nonexistant template %s"), $params{template});
+			error sprintf(gettext("nonexistant template %s"), $params{template});
 		}
 		my $template=HTML::Template->new(@params) unless $raw;
 	
@@ -248,9 +342,11 @@ sub preprocess_inline (@) { #{{{
 					my $content=get_inline_content($page, $params{destpage});
 					$template->param(content => $content);
 				}
-				$template->param(pageurl => urlto(bestlink($params{page}, $page), $params{destpage}));
+				$template->param(pageurl => urlto($page, $params{destpage}));
+				$template->param(inlinepage => $page);
 				$template->param(title => pagetitle(basename($page)));
 				$template->param(ctime => displaytime($pagectime{$page}, $params{timeformat}));
+				$template->param(mtime => displaytime($pagemtime{$page}, $params{timeformat}));
 				$template->param(first => 1) if $page eq $list[0];
 				$template->param(last => 1) if $page eq $list[$#list];
 	
@@ -297,30 +393,26 @@ sub preprocess_inline (@) { #{{{
 		}
 	}
 	
-	if ($feeds) {
-		if (exists $params{feedpages}) {
-			@feedlist=grep { pagespec_match($_, $params{feedpages}, location => $params{page}) } @feedlist;
-		}
-	
+	if ($feeds && ($emptyfeeds || @feedlist)) {
 		if ($rss) {
-			my $rssp=rsspage($params{destpage}).$feednum;
+			my $rssp=$feedbase."rss".$feednum;
 			will_render($params{destpage}, $rssp);
 			if (! $params{preview}) {
 				writefile($rssp, $config{destdir},
 					genfeed("rss",
-						$config{url}."/".rsspage($params{destpage}).$feednum, $desc, $params{guid}, $params{destpage}, @feedlist));
+						$config{url}."/".$rssp, $desc, $params{guid}, $params{destpage}, @feedlist));
 				$toping{$params{destpage}}=1 unless $config{rebuild};
-				$feedlinks{$params{destpage}}=qq{<link rel="alternate" type="application/rss+xml" title="RSS" href="$rssurl" />};
+				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/rss+xml" title="$desc (RSS)" href="$rssurl" />};
 			}
 		}
 		if ($atom) {
-			my $atomp=atompage($params{destpage}).$feednum;
+			my $atomp=$feedbase."atom".$feednum;
 			will_render($params{destpage}, $atomp);
 			if (! $params{preview}) {
 				writefile($atomp, $config{destdir},
-					genfeed("atom", $config{url}."/".atompage($params{destpage}).$feednum, $desc, $params{guid}, $params{destpage}, @feedlist));
+					genfeed("atom", $config{url}."/".$atomp, $desc, $params{guid}, $params{destpage}, @feedlist));
 				$toping{$params{destpage}}=1 unless $config{rebuild};
-				$feedlinks{$params{destpage}}=qq{<link rel="alternate" type="application/atom+xml" title="Atom" href="$atomurl" />};
+				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/atom+xml" title="$desc (Atom)" href="$atomurl" />};
 			}
 		}
 	}
@@ -328,18 +420,18 @@ sub preprocess_inline (@) { #{{{
 	return $ret if $raw || $nested;
 	push @inline, $ret;
 	return "<div class=\"inline\" id=\"$#inline\"></div>\n\n";
-} #}}}
+}
 
-sub pagetemplate_inline (@) { #{{{
+sub pagetemplate_inline (@) {
 	my %params=@_;
 	my $page=$params{page};
 	my $template=$params{template};
 
 	$template->param(feedlinks => $feedlinks{$page})
 		if exists $feedlinks{$page} && $template->query(name => "feedlinks");
-} #}}}
+}
 
-sub get_inline_content ($$) { #{{{
+sub get_inline_content ($$) {
 	my $page=shift;
 	my $destpage=shift;
 	
@@ -358,9 +450,9 @@ sub get_inline_content ($$) { #{{{
 	else {
 		return "";
 	}
-} #}}}
+}
 
-sub date_822 ($) { #{{{
+sub date_822 ($) {
 	my $time=shift;
 
 	my $lc_time=POSIX::setlocale(&POSIX::LC_TIME);
@@ -368,9 +460,9 @@ sub date_822 ($) { #{{{
 	my $ret=POSIX::strftime("%a, %d %b %Y %H:%M:%S %z", localtime($time));
 	POSIX::setlocale(&POSIX::LC_TIME, $lc_time);
 	return $ret;
-} #}}}
+}
 
-sub date_3339 ($) { #{{{
+sub date_3339 ($) {
 	my $time=shift;
 
 	my $lc_time=POSIX::setlocale(&POSIX::LC_TIME);
@@ -378,9 +470,9 @@ sub date_3339 ($) { #{{{
 	my $ret=POSIX::strftime("%Y-%m-%dT%H:%M:%SZ", gmtime($time));
 	POSIX::setlocale(&POSIX::LC_TIME, $lc_time);
 	return $ret;
-} #}}}
+}
 
-sub absolute_urls ($$) { #{{{
+sub absolute_urls ($$) {
 	# sucky sub because rss sucks
 	my $content=shift;
 	my $baseurl=shift;
@@ -401,17 +493,9 @@ sub absolute_urls ($$) { #{{{
 	$content=~s/(<a(?:\s+(?:class|id)\s*="?\w+"?)?)\s+href=\s*"(?!\w+:)(\/[^"]*)"/$1 href="$urltop$2"/mig;
 	$content=~s/(<img(?:\s+(?:class|id|width|height)\s*="?\w+"?)*)\s+src=\s*"(?!\w+:)(\/[^"]*)"/$1 src="$urltop$2"/mig;
 	return $content;
-} #}}}
+}
 
-sub rsspage ($) { #{{{
-	return targetpage(shift, "rss");
-} #}}}
-
-sub atompage ($) { #{{{
-	return targetpage(shift, "atom");
-} #}}}
-
-sub genfeed ($$$$$@) { #{{{
+sub genfeed ($$$$$@) {
 	my $feedtype=shift;
 	my $feedurl=shift;
 	my $feeddesc=shift;
@@ -438,9 +522,15 @@ sub genfeed ($$$$$@) { #{{{
 			mdate_3339 => date_3339($pagemtime{$p}),
 		);
 
-		if (exists $pagestate{$p} &&
-		    exists $pagestate{$p}{meta}{guid}) {
-			$itemtemplate->param(guid => $pagestate{$p}{meta}{guid});
+		if (exists $pagestate{$p}) {
+			if (exists $pagestate{$p}{meta}{guid}) {
+				$itemtemplate->param(guid => $pagestate{$p}{meta}{guid});
+			}
+
+			if (exists $pagestate{$p}{meta}{updated}) {
+				$itemtemplate->param(mdate_822 => date_822($pagestate{$p}{meta}{updated}));
+				$itemtemplate->param(mdate_3339 => date_3339($pagestate{$p}{meta}{updated}));
+			}
 		}
 
 		if ($itemtemplate->query(name => "enclosure")) {
@@ -496,9 +586,9 @@ sub genfeed ($$$$$@) { #{{{
 	});
 	
 	return $template->output;
-} #}}}
+}
 
-sub pingurl (@) { #{{{
+sub pingurl (@) {
 	return unless @{$config{pingurl}} && %toping;
 
 	eval q{require RPC::XML::Client};
@@ -530,20 +620,20 @@ sub pingurl (@) { #{{{
 					$title, $url);
 				my $res = $client->send_request($req);
 				if (! ref $res) {
-					debug("Did not receive response to ping");
+					error("Did not receive response to ping");
 				}
 				my $r=$res->value;
 				if (! exists $r->{flerror} || $r->{flerror}) {
-					debug("Ping rejected: ".(exists $r->{message} ? $r->{message} : "[unknown reason]"));
+					error("Ping rejected: ".(exists $r->{message} ? $r->{message} : "[unknown reason]"));
 				}
 			};
 			if ($@) {
-				debug "Ping failed: $@";
+				error "Ping failed: $@";
 			}
 		}
 	}
 
 	exit 0; # daemon done
-} #}}}
+}
 
 1
